@@ -1,20 +1,12 @@
-import six
-
-if six.PY3:
-    long = int
-else:
-    long = long
-
-
 def b(arg):
     """Convert `arg` to bytes."""
-    if isinstance(arg, six.text_type):
+    if isinstance(arg, str):
         arg = arg.encode("latin-1")
     return arg
 
 
 def u(arg):
     """Convert `arg` to text."""
-    if isinstance(arg, six.binary_type):
+    if isinstance(arg, bytes):
         arg = arg.decode('ascii', 'replace')
     return arg
